/*
 *  Copyright 2010 Anyware Services
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */

Ext.namespace('org.ametys.cms.ribbon.tab');

/**
 * Do not call this contructor
 * @constructor
 * @class This class handles the tab that shows when selection is of the configured HTML node type in the edition form
 * @param {String} id The unique tab identifier
 * @param {Map<id, value>} configuration The configuration of the control instance. The key 'node-type' represents the name of the HTML node that will make the tab show when this kind of target is in the {@see org.ametys.ribbon.RibbonManager.EVENTTYPE_SELECTIONCHANGED} message.
 * @param {String} The name of the plugin where the control was declared
 * @param {Ext.Panel} The panel of the tab
 */
org.ametys.cms.ribbon.tab.EditorTab = function(id, configuration, pluginName, tabPanel) 
{
	org.ametys.cms.ribbon.tab.EditorTab.superclass.constructor.call(this, id, configuration, pluginName, tabPanel);
};

Ext.extend(org.ametys.cms.ribbon.tab.EditorTab, org.ametys.ribbon.control.RibbonTab, {});

/**
 * @private
 * Get the node where to check attributes on
 */
org.ametys.cms.ribbon.tab.EditorTab.prototype._getNode = function(base)
{
	return base;
}

/**
 * Activate or deactivate the tab depending on the selection
 * @param {org.ametys.messagebus.message.Message[]} messages The messages received.
 * @private
 */
org.ametys.cms.ribbon.tab.EditorTab.prototype.onMessage = function (messages)
{
	for (var i=0; i < messages.length; i++)
	{
		if (messages[i].getType().getName() == org.ametys.ribbon.RibbonManager.EVENTTYPE_SELECTIONCHANGED)
		{
			var targets = messages[i].getTargets();
			var target = this._searchNodeTarget(targets);

			if (target != null && tinyMCE.activeEditor != null)
			{
				var node = tinyMCE.activeEditor.dom.getParent(this._getNode(target.getParameters()['object']), this.getConfiguration()['node-type']);

				while (node != null)
				{
					var show = true;
					var attributes = this.getConfiguration()['attribute'];
					var attributesAsArray = attributes != null ? attributes.split(";") : [];
					for (var j=0; show && j < attributesAsArray.length; j++)
					{
						var attribute = attributesAsArray[j];
						
						var reverse = false;
						if (attribute != null && attribute.length > 1 && attribute.charAt(0) == "!")
						{
							attribute = attribute.substring(1);
							reverse = true;
						}
	
						if (!(attribute == null || attribute == "" 
							|| (!reverse && node.getAttribute(attribute) != null)
							|| (reverse && node.getAttribute(attribute) == null)))
						{
							show = false;
						}
					}
					
					if (show)
					{
						var shouldForce = node.getAttribute("_mce_ribbon_select") == "1"; 
						this.show(shouldForce);
						
						if (shouldForce)
						{
							// disarm the force flag
							var a = tinyMCE.activeEditor.dom.setAttrib.createDelegate(tinyMCE.activeEditor.dom, [node, "_mce_ribbon_select", '']);
							// do it lately so brother tabs will receive the flag also
							window.setTimeout(a, 1);
						}
						return;
					}
					
					node = tinyMCE.activeEditor.dom.getParent(node.parentNode, this.getConfiguration()['node-type']);
				}
			}
			
			this.hide();
		}
	}
}

/**
 * @private
 * Search in an array of targets for a content target
 * @param {org.ametys.messagebus.message.MessageTarget[]} targets The array where to search
 * @return {org.ametys.messagebus.message.MessageTarget} The content found
 */
org.ametys.cms.ribbon.tab.EditorTab.prototype._searchNodeTarget = function(targets)
{
	var contentTarget = org.ametys.messagebus.message.MessageTargetHelper.findFirst(targets, function(target) { return target.getType() == "content";});
	if (contentTarget != null)
	{
		var formTarget = org.ametys.messagebus.message.MessageTargetHelper.findFirst(contentTarget.getSubTargets(), function(target) { return target.getType() == "form";});
		if (formTarget != null)
		{
			var fieldTarget = org.ametys.messagebus.message.MessageTargetHelper.findFirst(formTarget.getSubTargets(), function(target) { return target.getType() == "field";});
			if (fieldTarget != null)
			{
				return org.ametys.messagebus.message.MessageTargetHelper.findFirst(fieldTarget.getSubTargets(), function(target) { return target.getType() == "node";});
			}
			
		}
	}
	return null;
};
