/*
 *  Copyright 2010 Anyware Services
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */

Ext.namespace('org.ametys.ribbon.control.button');

/**
 * ActionButton that moreover is disabled when the defined 'target-type' is not selected
 * The configuration of the control instance. The key 'target-type' represents the name of the target that will make the button enable when this kind of target is in the {@see org.ametys.ribbon.RibbonManager.EVENTTYPE_SELECTIONCHANGED} message. You can optionnaly set a subtarget-type that do the same on the immediate first child
 */
org.ametys.ribbon.control.button.ActionSelectionButton = function(id, configuration, pluginName)
{
	org.ametys.ribbon.control.button.ActionSelectionButton.superclass.constructor.call(this, id, configuration, pluginName);

	this._targetRegExp = new RegExp(configuration['target-type']);
	if (configuration['subtarget-type'] != null)
	{
		this._subtargetRegExp = new RegExp(configuration['subtarget-type']);
	}
	
	if (configuration['subsubtarget-type'] != null)
	{
		this._subsubtargetRegExp = new RegExp(configuration['subsubtarget-type']);
	}
}

Ext.extend(org.ametys.ribbon.control.button.ActionSelectionButton, org.ametys.ribbon.control.button.ActionButton, {});

/**
 * @private
 * @property {RegExp} _targetRegExp The regexp for the target lookup
 */
org.ametys.ribbon.control.button.ActionSelectionButton.prototype._targetRegExp;

/**
 * @private
 * @property {RegExp} _subtargetRegExp The regexp for the sub target lookup
 */
org.ametys.ribbon.control.button.ActionSelectionButton.prototype._subtargetRegExp;

/**
 * @private
 * @property {RegExp} _subtargetRegExp The regexp for the sub sub target lookup
 */
org.ametys.ribbon.control.button.ActionSelectionButton.prototype._subsubtargetRegExp;

org.ametys.ribbon.control.button.ActionSelectionButton.prototype.onMessage = function(messages)
{
	for (var i=0; i < messages.length; i++)
	{
		if (messages[i].getType().getName() == org.ametys.ribbon.RibbonManager.EVENTTYPE_SELECTIONCHANGED)
		{
			var allTargets = messages[i].getTargets();
			var targets = org.ametys.messagebus.message.MessageTargetHelper.findAll(allTargets, this._testTarget.createDelegate(this));

			if (targets.length == 1 || targets.length > 1 && this.getConfiguration()['enable-multiselection'] != "false")
			{
				this.getElements().each(function(element) { element.enable(); });
			}
			else
			{
				this.getElements().each(function(element) { element.disable(); });
			}
		}
	}
}

/**
 * @private
 * Test a target. Does it match ?
 * @return {org.ametys.messagebus.message.MessageTarget} The target to check
 */
org.ametys.ribbon.control.button.ActionSelectionButton.prototype._testTarget = function(target)
{
	if (!this._targetRegExp.test(target.getType()))
	{
		// the main target does not match
		return false;
	}
	
	if (this._subtargetRegExp == null)
	{
		// no need to check sub-targets
		return true;
	}
	
	// check sub-targets
	var subTargets = target.getSubTargets();
	for (var i=0; i < subTargets.length; i++)
	{
		if (this._subtargetRegExp.test(subTargets[i].getType()))
		{
			// at least a sub-target matches
			
			if (this._subsubtargetRegExp == null)
			{
				// no need to check sub-sub-targets
				return true;
			}
			
			// check the sub-sub-targets
			var subsubTargets = subTargets[i].getSubTargets();
			for (var j=0; j < subsubTargets.length; j++)
			{
				if (this._subsubtargetRegExp.test(subsubTargets[j].getType()))
				{
					// at least a sub-sub-target matches
					return true;
				}
			}
		}
	}
}
