/*
 *  Copyright 2010 Anyware Services
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */

Ext.namespace('org.ametys.uitool.tool');

/**
 * UI interface for tools extends <a href="http://extjs.com/deploy/dev/docs/?class=Ext.TabPanel">Ext.TabPanel</a>
 * @class This class represents the ui interface for tools.
 * @param config Configuration options
 * @constructor
 */
org.ametys.uitool.tool.ToolPanel = function(config) 
{
	org.ametys.uitool.tool.ToolPanel.superclass.constructor.call(this, config);
}; 

/**
 * @constant The default tabs width
 * @private
 */
org.ametys.uitool.tool.ToolPanel.TAB_WIDTH = 250;

/**
 * @private
 * @static
 * @property {Ext.Panel} _focusedPanel The currently focused panel
 */
org.ametys.uitool.tool.ToolPanel._focusedPanel;

/**
 * This method is called when the panel element (not the panel itself) may have received the focus.
 * The scope must be the panel.
 * @static
 * @private
 */
org.ametys.uitool.tool.ToolPanel.gotElFocus = function()
{
	var p = this;
	while (!p.ownerCt.toolPanel)
	{
		p = p.ownerCt;
	}
	org.ametys.uitool.tool.ToolPanel.gotFocus(p);
}

/**
 * This method is called when a panel may have received the focus.
 * The 'x-tab-strip-focus' class is added to the strip
 * @private
 * @static
 * @param {Ext.Panel} panel The panel involved
 */
org.ametys.uitool.tool.ToolPanel.gotFocus = function(panel)
{
	if (org.ametys.uitool.tool.ToolPanel._focusedPanel != panel)
	{
		org.ametys.uitool.tool.ToolPanel.lostFocus(org.ametys.uitool.tool.ToolPanel._focusedPanel);
		
		org.ametys.uitool.tool.ToolPanel._focusedPanel = panel;
		Ext.get(panel.ownerCt.getTabEl(panel).id).addClass("x-tab-strip-focus");
    	panel.fireEvent('focus', panel);
	}
}

/**
 * This method is called when a panel may have received the focus
 * The 'x-tab-strip-focus' class is remove from the strip
 * @private
 * @static
 * @param {Ext.Panel} panel The panel involved
 */
org.ametys.uitool.tool.ToolPanel.lostFocus = function(panel)
{
	if (org.ametys.uitool.tool.ToolPanel._focusedPanel == panel && org.ametys.uitool.tool.ToolPanel._focusedPanel != null)
	{
		if (panel.ownerCt)
		{
			Ext.get(panel.ownerCt.getTabEl(panel).id).removeClass("x-tab-strip-focus");
	    }

		panel.fireEvent('blur', panel);
    	
	    org.ametys.uitool.tool.ToolPanel._focusedPanel = null;
    }
}

Ext.extend(org.ametys.uitool.tool.ToolPanel, Ext.TabPanel, 
{
	resizeTabs:true, // turn on tab resizing
	closable : true,
	toolPanel: true,
	border: false,
	activeTab: 0,
	deferredRender: false,
	ametysCls: 'ametys',
	layoutOnTabChange: true,
	tabWidth : org.ametys.uitool.tool.ToolPanel.TAB_WIDTH
});

org.ametys.uitool.tool.ToolPanel.prototype.onRender = function(ct, position)
{
	org.ametys.uitool.tool.ToolPanel.superclass.onRender.call(this, ct, position);
	
	this.header.addClass(this.ametysCls + '-tool-panel-header');
	this.body.addClass(this.ametysCls + '-tool-panel-body');
}

//override, to add listeners on adding and removing panels
org.ametys.uitool.tool.ToolPanel.prototype.initComponent = function()
{
	this.addListener('add', this.onAddPanelForFocus, this);
	this.addListener('remove', this.onRemovePanelForFocus, this);

	this.addListener('add', this.onAddPanelForShow, this);
	this.addListener('remove', this.onRemovePanelForShow, this);

    org.ametys.uitool.tool.ToolPanel.superclass.initComponent.call(this);
}

// override to handle when clicking on tab stip (when already selected)
org.ametys.uitool.tool.ToolPanel.prototype.onStripMouseDown = function(e)
{
    var t = this.findTargets(e);
    if(t.close == null && t.item != null && t.item == this.activeTab)
    {
    	var panel = this.getComponent(t.item);
    	org.ametys.uitool.tool.ToolPanel.gotFocus(panel);
    }

    if(e.button != 0)
    {
        return;
    }
    e.preventDefault();
    if(t.close)
    {
    	var panel = this.getComponent(t.item);

    	if (panel.fireEvent('manualclose', this, panel))
    	{
    		this.remove(t.item);
    	}
        return;
    }
    if(t.item && t.item != this.activeTab)
    {
        this.setActiveTab(t.item);
    }
}

/**
 * This method is called when a panel is added to this tabpanel.
 * Its behavior is to show the tabpanel
 * @private
 * @param {Ext.TabPanel} panel tabpanel The parent tab panel
 * @param {Ext.Panel} panel panel The panel to modify
 */
org.ametys.uitool.tool.ToolPanel.prototype.onAddPanelForShow = function(tabpanel, panel)
{ 
	if (this != tabpanel)
	{
		return;
	}

    panel.ownerCt.doLayout(null, true);

	if (!tabpanel.ownerCt.isVisible())
	{
		tabpanel.ownerCt.show();
		tabpanel.ownerCt.ownerCt.doLayout();
	}
}
/**
 * This method is called when the panel is beeing removed to hide it
 * @private
 * @param {Ext.TabPanel} panel tabpanel The parent tab panel
 */
org.ametys.uitool.tool.ToolPanel.prototype.onRemovePanelForShow = function(tabpanel)
{
	if (this != tabpanel)
	{
		return;
	}
	
	if (tabpanel.items.length == 0)
	{
		tabpanel.ownerCt.hide();
		tabpanel.ownerCt.ownerCt.doLayout();
	}
}

/**
 * This method is call when a panel is added to this tabpanel.
 * Its behavior is to register several events in order to fire the 'focus' event on the given panel
 * @private
 * @param {Ext.TabPanel} panel tabpanel The parent tab panel
 * @param {Ext.Panel} panel panel The panel to modify
 */
org.ametys.uitool.tool.ToolPanel.prototype.onAddPanelForFocus = function(tabpanel, panel)
{
	if (this != tabpanel)
	{
		return;
	}

	panel.addEvents(
		'focus', 
		'blur'
		);

	panel.addListener('render', this.onPanelRenderedForFocus, panel);

	panel.addListener('activate', org.ametys.uitool.tool.ToolPanel.gotFocus, panel);
	panel.addListener('deactivate', org.ametys.uitool.tool.ToolPanel.lostFocus, panel);
	panel.addListener('close', org.ametys.uitool.tool.ToolPanel.lostFocus, panel);
	panel.addListener('hide', org.ametys.uitool.tool.ToolPanel.lostFocus, panel);
	panel.addListener('collapse', org.ametys.uitool.tool.ToolPanel.lostFocus, panel);
	panel.addListener('destroy', org.ametys.uitool.tool.ToolPanel.lostFocus, panel);
	panel.addListener('disable', org.ametys.uitool.tool.ToolPanel.lostFocus, panel);
}

/**
 * The method completes the {@link org.ametys.uitool.tool.ToolPanel#onAddPanelForFocus} by registering several events on the main element of the panel
 * @private
 * @param {Ext.Panel} panel The panel to modify
 */
org.ametys.uitool.tool.ToolPanel.prototype.onPanelRenderedForFocus = function(panel)
{
	panel.el.addListener('click', org.ametys.uitool.tool.ToolPanel.gotElFocus, panel);
	panel.el.addListener('keydown', org.ametys.uitool.tool.ToolPanel.gotElFocus, panel);
}

/**
 * This method is called when the panel is beeing removerd to unregister every listener
 * @private
 * @param {Ext.Panel} panel The panel to modify
 */
org.ametys.uitool.tool.ToolPanel.prototype.onRemovePanelForFocus = function(panel)
{
	if (panel.isRendered)
	{
		panel.el.removeListener('click', org.ametys.uitool.tool.ToolPanel.gotElFocus, panel);
		panel.el.removeListener('keydown', org.ametys.uitool.tool.ToolPanel.gotElFocus, panel);
	}
	panel.removeListener('activate', org.ametys.uitool.tool.ToolPanel.gotFocus, panel);
	
	panel.removeListener('deactivate', org.ametys.uitool.tool.ToolPanel.lostFocus, panel);
	panel.removeListener('close', org.ametys.uitool.tool.ToolPanel.lostFocus, panel);
	panel.removeListener('hide', org.ametys.uitool.tool.ToolPanel.lostFocus, panel);
	panel.removeListener('collapse', org.ametys.uitool.tool.ToolPanel.lostFocus, panel);
	panel.removeListener('destroy', org.ametys.uitool.tool.ToolPanel.lostFocus, panel);
	panel.removeListener('disable', org.ametys.uitool.tool.ToolPanel.lostFocus, panel);
}

/**
 * Creates the panel associated with a tool
 * @param {org.ametys.uitool.tool.Tool} tool The tool to add
 */
org.ametys.uitool.tool.ToolPanel.prototype.addUITool = function (tool)
{
	var oodPanel = tool.getOutOfPanel();
	var content = tool.createPanel();
	content.region = "center";
	
	var panel = new Ext.Panel ({
		layout: 'border',
		closable: true,
		border: false,
		autoScroll: true,
		iconCls: "instyleicon",
		items : [content, oodPanel],
		oodPanel : oodPanel,
		onManualClose : function (p)
		{
			org.ametys.uitool.ToolsManager.getInstance().getTool(tool.getId()).close();
			//cancel the close
			return false;
		}
	});
	panel.addListener('manualclose', panel.onManualClose);
	content.relayEvents(panel, ['focus', 'blur', 'activate', 'deactivate', 'close']);

	panel.addListener('focus', function () { this.onEvent(this, org.ametys.ribbon.RibbonManager.EVENTTYPE_TOOLFOCUSES); }, tool);
	panel.addListener('blur', function () { this.onEvent(this, org.ametys.ribbon.RibbonManager.EVENTTYPE_TOOLBLUR); }, tool);
	panel.addListener('activate', function () { this.onEvent(this, org.ametys.ribbon.RibbonManager.EVENTTYPE_TOOLACTIVATED); }, tool);
	panel.addListener('deactivate', function () { this.onEvent(this, org.ametys.ribbon.RibbonManager.EVENTTYPE_TOOLDEACTIVATED); }, tool);
	panel.addListener('open', function () { this.onEvent(this, org.ametys.ribbon.RibbonManager.EVENTTYPE_TOOLOPENED); }, tool);
	panel.addListener('close', function () { this.onEvent(this, org.ametys.ribbon.RibbonManager.EVENTTYPE_TOOLCLOSED); }, tool);
	
	panel.fireEvent('open', panel, panel);
	
	tool.toolsPanel = this;
	tool.parent = panel;
	tool._repaintTooltip();
	
	this.add(panel);

    var el = this.getTabEl(panel);
    if(el)
    {
        Ext.fly(el).child('span.x-tab-strip-text').dom.style.backgroundImage = "url('" + context.contextPath + tool._iconSmall + "')";
    }	
}

